@@ -39,6 +39,10 @@
  *---------------------------------------------------------------*/
 
 
+#include "direwolf.h"		// Sets _WIN32_WINNT for XP API level needed by ws2tcpip.h
+				// Also includes windows.h.
+
+
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -48,7 +52,6 @@
 #include <io.h>
 #include <fcntl.h>
 
-#include <windows.h>		
 #include <mmsystem.h>
 
 #ifndef WAVE_FORMAT_96M16
@@ -57,11 +60,9 @@
 #endif
 
 #include <winsock2.h>
-#define _WIN32_WINNT 0x0501
-#include <ws2tcpip.h>
+#include <ws2tcpip.h>  		// _WIN32_WINNT must be set to 0x0501 before including this
 
 
-#include "direwolf.h"
 #include "audio.h"
 #include "audio_stats.h"
 #include "textcolor.h"
@@ -286,7 +287,7 @@ int audio_open (struct audio_s *pa)
 
 	    A->udp_sock = INVALID_SOCKET;
 
-	    in_dev_no[a] = WAVE_MAPPER;	/* = -1 */
+	    in_dev_no[a] = WAVE_MAPPER;	/* = ((UINT)-1) in mmsystem.h */
 	    out_dev_no[a] = WAVE_MAPPER;
 
 /*
@@ -312,23 +313,28 @@ int audio_open (struct audio_s *pa)
 
 	      /* Does config file have a number?  */
 	      /* If so, it is an index into list of devices. */
+	      /* Originally only a single digit was recognized.  */
+	      /* v 1.5 also recognizes two digits.  (Issue 116) */
 
 	      if (strlen(pa->adev[a].adevice_in) == 1 && isdigit(pa->adev[a].adevice_in[0])) {
 	        in_dev_no[a] = atoi(pa->adev[a].adevice_in);
 	      }
+	      else if (strlen(pa->adev[a].adevice_in) == 2 && isdigit(pa->adev[a].adevice_in[0]) && isdigit(pa->adev[a].adevice_in[1])) {
+	        in_dev_no[a] = atoi(pa->adev[a].adevice_in);
+	      }
 
 	      /* Otherwise, does it have search string? */
 
-	      if (in_dev_no[a] == WAVE_MAPPER && strlen(pa->adev[a].adevice_in) >= 1) {
+	      if ((UINT)(in_dev_no[a]) == WAVE_MAPPER && strlen(pa->adev[a].adevice_in) >= 1) {
 	        num_devices = waveInGetNumDevs();
-	        for (n=0 ; n<num_devices && in_dev_no[a] == WAVE_MAPPER ; n++) {
+	        for (n=0 ; n<num_devices && (UINT)(in_dev_no[a]) == WAVE_MAPPER ; n++) {
 	          if ( ! waveInGetDevCaps(n, &wic, sizeof(WAVEINCAPS))) {
 	            if (strstr(wic.szPname, pa->adev[a].adevice_in) != NULL) {
 	              in_dev_no[a] = n;
 	            }
 	          }
 	        }
-	        if (in_dev_no[a] == WAVE_MAPPER) {
+	        if ((UINT)(in_dev_no[a]) == WAVE_MAPPER) {
 	          text_color_set(DW_COLOR_ERROR);
 	          dw_printf ("\"%s\" doesn't match any of the input devices.\n", pa->adev[a].adevice_in);
 	        }
@@ -343,17 +349,20 @@ int audio_open (struct audio_s *pa)
 	    if (strlen(pa->adev[a].adevice_out) == 1 && isdigit(pa->adev[a].adevice_out[0])) {
 	      out_dev_no[a] = atoi(pa->adev[a].adevice_out);
 	    }
+	    else if (strlen(pa->adev[a].adevice_out) == 2 && isdigit(pa->adev[a].adevice_out[0]) && isdigit(pa->adev[a].adevice_out[1])) {
+	      out_dev_no[a] = atoi(pa->adev[a].adevice_out);
+	    }
 
-	    if (out_dev_no[a] == WAVE_MAPPER && strlen(pa->adev[a].adevice_out) >= 1) {
+	    if ((UINT)(out_dev_no[a]) == WAVE_MAPPER && strlen(pa->adev[a].adevice_out) >= 1) {
 	      num_devices = waveOutGetNumDevs();
-	      for (n=0 ; n<num_devices && out_dev_no[a] == WAVE_MAPPER ; n++) {
+	      for (n=0 ; n<num_devices && (UINT)(out_dev_no[a]) == WAVE_MAPPER ; n++) {
 	        if ( ! waveOutGetDevCaps(n, &woc, sizeof(WAVEOUTCAPS))) {
 	          if (strstr(woc.szPname, pa->adev[a].adevice_out) != NULL) {
 	            out_dev_no[a] = n;
 	          }
 	        }
 	      }
-	      if (out_dev_no[a] == WAVE_MAPPER) {
+	      if ((UINT)(out_dev_no[a]) == WAVE_MAPPER) {
 	        text_color_set(DW_COLOR_ERROR);
 	        dw_printf ("\"%s\" doesn't match any of the output devices.\n", pa->adev[a].adevice_out);
 	      }
@@ -798,7 +807,7 @@ int audio_get (int a)
 
 	      p = (WAVEHDR*)(A->in_headp);		/* no need to be volatile at this point */
 
-	      if (p->dwUser == -1) {
+	      if (p->dwUser == (DWORD)(-1)) {
 	        waveInUnprepareHeader(A->audio_in_handle, p, sizeof(WAVEHDR));
 	        p->dwUser = 0;	/* Index for next byte. */
 
@@ -841,7 +850,7 @@ int audio_get (int a)
 
               assert (A->udp_sock > 0);
 
-	      res = recv (A->udp_sock, A->stream_data, SDR_UDP_BUF_MAXLEN, 0);
+	      res = SOCK_RECV (A->udp_sock, A->stream_data, SDR_UDP_BUF_MAXLEN);
 	      if (res <= 0) {
 	        text_color_set(DW_COLOR_ERROR);
 	        dw_printf ("Can't read from udp socket, errno %d", WSAGetLastError());
@@ -942,7 +951,15 @@ int audio_put (int a, int c)
 	  timeout--;
 	  if (timeout <= 0) {
 	    text_color_set(DW_COLOR_ERROR);
+
+// TODO: open issues 78 & 165.  How can we avoid/improve this?
+
 	    dw_printf ("Audio output failure waiting for buffer.\n");
+	    dw_printf ("This can occur when we are producing audio output for\n");
+	    dw_printf ("transmit and the operating system doesn't provide buffer\n");
+	    dw_printf ("space after waiting and retrying many times.\n");
+	    //dw_printf ("In recent years, this has been reported only when running the\n");
+	    //dw_printf ("Windows version with VMWare on a Macintosh.\n");
 	    ptt_term ();
 	    return (-1);
 	  }
@@ -959,11 +976,11 @@ int audio_put (int a, int c)
 	/* Should never be full at this point. */
 
 	assert (p->dwBufferLength >= 0);
-	assert (p->dwBufferLength < A->outbuf_size);
+	assert (p->dwBufferLength < (DWORD)(A->outbuf_size));
 
 	p->lpData[p->dwBufferLength++] = c;
 
-	if (p->dwBufferLength == A->outbuf_size) {
+	if (p->dwBufferLength == (DWORD)(A->outbuf_size)) {
 	  return (audio_flush(a));
 	}
 
