@@ -34,13 +34,24 @@ enum sendto_type_e { SENDTO_XMIT, SENDTO_IGATE, SENDTO_RECV };
 struct misc_config_s {
 
 	int agwpe_port;		/* Port number for the "AGW TCPIP Socket Interface" */
-	int kiss_port;		/* Port number for the "KISS" protocol. */
+	int kiss_port;		/* Port number for the "TCP KISS" protocol. */
 	int enable_kiss_pt;	/* Enable pseudo terminal for KISS. */
 				/* Want this to be off by default because it hangs */
 				/* after a while if nothing is reading from other end. */
 
-	char nullmodem[20];	/* Serial port name for our end of the */
+	char kiss_serial_port[20];
+				/* Serial port name for our end of the */
 				/* virtual null modem for native Windows apps. */
+				/* Version 1.5 add same capability for Linux. */
+
+	int kiss_serial_speed;	/* Speed, in bps, for the KISS serial port. */
+				/* If 0, just leave what was already there. */
+
+	int kiss_serial_poll;	/* When using Bluetooth KISS, the /dev/rfcomm0 device */
+				/* will appear and disappear as the remote application */
+				/* opens and closes the virtual COM port. */
+				/* When this is non-zero, we will check periodically to */
+				/* see if the device has appeared and we will open it. */
 
 	char gpsnmea_port[20];	/* Serial port name for reading NMEA sentences from GPS. */
 				/* e.g. COM22, /dev/ttyACM0 */
@@ -66,7 +77,9 @@ struct misc_config_s {
 #define WPT_FORMAT_KENWOOD      0x08		/* K	$PKWDWPL */
 
 
-	char logdir[80];	/* Directory for saving activity logs. */
+	int log_daily_names;	/* True to generate new log file each day. */
+
+	char log_path[80];	/* Either directory or full file name depending on above. */
 
 	int sb_configured;	/* TRUE if SmartBeaconing is configured. */
 	int sb_fast_speed;	/* MPH */
@@ -93,6 +106,17 @@ struct misc_config_s {
 				/* switching to SABM.  This is to handle the case of an old */
 				/* TNC which simply ignores SABME rather than replying with FRMR. */
 
+	char **v20_addrs;	/* Stations known to understand only AX.25 v2.0 so we don't */
+				/* waste time trying v2.2 first. */
+
+	int v20_count;		/* Number of station addresses in array above. */
+
+	char **noxid_addrs;	/* Stations known not to understand XID command so don't */
+				/* waste time sending it and eventually giving up. */
+				/* AX.25 for Linux is the one known case, so far, where */
+				/* SABME is implemented but XID is not. */
+
+	int noxid_count;	/* Number of station addresses in array above. */
 
 
 // Beacons.
@@ -120,6 +144,9 @@ struct misc_config_s {
 
 	  int delay;		/* Seconds to delay before first transmission. */
 
+	  int slot;		/* Seconds after hour for slotted time beacons. */
+				/* If specified, it overrides any 'delay' value. */
+
 	  int every;		/* Time between transmissions, seconds. */
 				/* Remains fixed for PBEACON and OBEACON. */
 				/* Dynamically adjusted for TBEACON. */
@@ -146,6 +173,7 @@ struct misc_config_s {
 
 	  double lat;		/* Latitude and longitude. */
 	  double lon;
+	  int ambiguity;	/* Number of lower digits to trim from location. 0 (default), 1, 2, 3, 4. */
 	  float alt_m;		/* Altitude in meters. */
 
 	  char symtab;		/* Symbol table: / or \ or overlay character. */
