@@ -3,14 +3,15 @@
 /* demod.h */
 
 #include "audio.h" 	/* for struct audio_s */
+#include "ax25_pad.h"	/* for alevel_t */
 
 
 int demod_init (struct audio_s *pa);
 
-int demod_get_sample (void);
+int demod_get_sample (int a);
 
 void demod_process_sample (int chan, int subchan, int sam);
 
 void demod_print_agc (int chan, int subchan);
 
-int demod_get_audio_level (int chan, int subchan);
\ No newline at end of file
+alevel_t demod_get_audio_level (int chan, int subchan);
\ No newline at end of file
