@@ -18,7 +18,7 @@
 //
 
 
-// #define DEBUG5 1	/* capture 9600 output to log files */
+//#define DEBUG4 1	/* capture 9600 output to log files */
 
 
 /*------------------------------------------------------------------
@@ -33,6 +33,8 @@
  *
  *---------------------------------------------------------------*/
 
+#include "direwolf.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
@@ -42,7 +44,6 @@
 #include <assert.h>
 #include <ctype.h>
 
-#include "direwolf.h"
 #include "tune.h"
 #include "fsk_demod_state.h"
 #include "hdlc_rec.h"
@@ -72,25 +73,10 @@ static inline float convolve (const float *__restrict__ data, const float *__res
 	float sum = 0.0f;
 	int j;
 
-#if 0
-	//  As suggested here,  http://locklessinc.com/articles/vectorize/
-	//  Unfortunately, older compilers don't recognize it.
-
-	//  Get more information by using -ftree-vectorizer-verbose=5
-
-	float *d = __builtin_assume_aligned(data, 16);
-	float *f = __builtin_assume_aligned(filter, 16);
-
-#pragma GCC ivdep
-	for (j=0; j<filter_size; j++) {
-	    sum += f[j] * d[j];
-	}
-#else
-#pragma GCC ivdep				// ignored until gcc 4.9
+//#pragma GCC ivdep				// ignored until gcc 4.9
 	for (j=0; j<filter_size; j++) {
 	    sum += filter[j] * data[j];
 	}
-#endif
 	return (sum);
 }
 
@@ -101,21 +87,21 @@ __attribute__((hot)) __attribute__((always_inline))
 static inline float agc (float in, float fast_attack, float slow_decay, float *ppeak, float *pvalley)
 {
 	if (in >= *ppeak) {
-	  *ppeak = in * fast_attack + *ppeak * (1. - fast_attack);
+	  *ppeak = in * fast_attack + *ppeak * (1.0f - fast_attack);
 	}
 	else {
-	  *ppeak = in * slow_decay + *ppeak * (1. - slow_decay);
+	  *ppeak = in * slow_decay + *ppeak * (1.0f - slow_decay);
 	}
 
 	if (in <= *pvalley) {
-	  *pvalley = in * fast_attack + *pvalley * (1. - fast_attack);
+	  *pvalley = in * fast_attack + *pvalley * (1.0f - fast_attack);
 	}
 	else  {   
-	  *pvalley = in * slow_decay + *pvalley * (1. - slow_decay);
+	  *pvalley = in * slow_decay + *pvalley * (1.0f - slow_decay);
 	}
 
 	if (*ppeak > *pvalley) {
-	  return ((in - 0.5 * (*ppeak + *pvalley)) / (*ppeak - *pvalley));
+	  return ((in - 0.5f * (*ppeak + *pvalley)) / (*ppeak - *pvalley));
 	}
 	return (0.0);
 }
@@ -125,11 +111,11 @@ static inline float agc (float in, float fast_attack, float slow_decay, float *p
  *
  * Name:        demod_9600_init
  *
- * Purpose:     Initialize the 9600 baud demodulator.
+ * Purpose:     Initialize the 9600 (or higher) baud demodulator.
  *
  * Inputs:      samples_per_sec	- Number of samples per second.
- *				Might be upsampled in hopes of 
- *				reducing the PLL jitter.
+ *				  Might be upsampled in hopes of
+ *				  reducing the PLL jitter.
  *
  *		baud		- Data rate in bits per second.
  *
@@ -147,21 +133,33 @@ void demod_9600_init (int samples_per_sec, int baud, struct demodulator_state_s
 	memset (D, 0, sizeof(struct demodulator_state_s));
 	D->num_slicers = 1;
 
-	//dw_printf ("demod_9600_init(rate=%d, baud=%d, D ptr)\n", samples_per_sec, baud);
+// Multiple profiles in future?
+
+//	switch (profile) {
+
+//	  case 'J':			// upsample x2 with filtering.
+//	  case 'K':			// upsample x3 with filtering.
+//	  case 'L':			// upsample x4 with filtering.
+
+	    D->lp_filter_len_bits =  76 * 9600.0 / (44100.0 * 2.0);
+
+	    // Works best with odd number in some tests.  Even is better in others.
+	    //D->lp_filter_size = ((int) (0.5f * ( D->lp_filter_len_bits * (float)samples_per_sec / (float)baud ))) * 2 + 1;
+	    D->lp_filter_size = (int) (( D->lp_filter_len_bits * (float)samples_per_sec / baud) + 0.5f);
+
+	    D->lp_window = BP_WINDOW_HAMMING;
+	    D->lpf_baud = 0.62;
+
+	    D->agc_fast_attack = 0.080;
+	    D->agc_slow_decay =  0.00012;
+
+	    D->pll_locked_inertia = 0.89;
+	    D->pll_searching_inertia = 0.67;
+//	    break;
+//	}
 
         D->pll_step_per_sample = 
 		(int) round(TICKS_PER_PLL_CYCLE * (double) baud / (double)samples_per_sec);
-	
-	D->lp_filter_len_bits =  72 * 9600.0 / (44100.0 * 2.0);		
-	D->lp_filter_size = (int) (( D->lp_filter_len_bits * (float)samples_per_sec / baud) + 0.5);
-	D->lp_window = BP_WINDOW_HAMMING;
-	D->lpf_baud = 0.59;	
-
-	D->agc_fast_attack = 0.080;	
-	D->agc_slow_decay = 0.00012;
-
-	D->pll_locked_inertia = 0.88;
-	D->pll_searching_inertia = 0.67;
 
 
 #ifdef TUNE_LP_WINDOW
@@ -184,8 +182,11 @@ void demod_9600_init (int samples_per_sec, int baud, struct demodulator_state_s
 	D->agc_slow_decay = TUNE_AGC_SLOW;
 #endif
 
-#if defined(TUNE_PLL_LOCKED) && defined(TUNE_PLL_SEARCHING)
+#if defined(TUNE_PLL_LOCKED)
 	D->pll_locked_inertia = TUNE_PLL_LOCKED;
+#endif
+
+#if defined(TUNE_PLL_SEARCHING)
 	D->pll_searching_inertia = TUNE_PLL_SEARCHING;
 #endif
 
@@ -198,7 +199,7 @@ void demod_9600_init (int samples_per_sec, int baud, struct demodulator_state_s
 	/* Version 1.2: Experiment with different slicing levels. */
 
 	for (j = 0; j < MAX_SUBCHANS; j++) {
-	  slice_point[j] = 0.02 * (j - 0.5 * (MAX_SUBCHANS-1));
+	  slice_point[j] = 0.02f * (j - 0.5f * (MAX_SUBCHANS-1));
 	  //dw_printf ("slice_point[%d] = %+5.2f\n", j, slice_point[j]);
 	}
 
@@ -260,23 +261,22 @@ void demod_9600_init (int samples_per_sec, int baud, struct demodulator_state_s
  *
  *--------------------------------------------------------------------*/
 
-static void inline nudge_pll (int chan, int subchan, int slice, int demod_data, struct demodulator_state_s *D);
+inline static void nudge_pll (int chan, int subchan, int slice, float demod_out, struct demodulator_state_s *D);
 
 __attribute__((hot))
 void demod_9600_process_sample (int chan, int sam, struct demodulator_state_s *D)
 {
 
 	float fsam;
-	//float abs_fsam;
 	float amp;
 	float demod_out;
 
-#if DEBUG5
+#if DEBUG4
 	static FILE *demod_log_fp = NULL;
-	static int seq = 0;			/* for log file name */
+	static int log_file_seq = 0;		/* Part of log file name */
 #endif
 
-	//int j;
+
 	int subchan = 0;
 	int demod_data;				/* Still scrambled. */
 
@@ -306,6 +306,12 @@ void demod_9600_process_sample (int chan, int sam, struct demodulator_state_s *D
 
 	fsam = sam / 16384.0;
 
+#if defined(TUNE_ZEROSTUFF) && TUNE_ZEROSTUFF == 0
+// experiment - no filtering.
+
+	amp = fsam;
+
+#else
 	push_sample (fsam, D->raw_cb, D->lp_filter_size);
 
 /*
@@ -313,7 +319,7 @@ void demod_9600_process_sample (int chan, int sam, struct demodulator_state_s *D
  */
 
 	amp = convolve (D->raw_cb, D->lp_filter, D->lp_filter_size);
-
+#endif
 
 /*
  * Version 1.2: Capture the post-filtering amplitude for display.
@@ -324,18 +330,20 @@ void demod_9600_process_sample (int chan, int sam, struct demodulator_state_s *D
  * Here we keep + and - peaks because there could be a DC bias.
  */
 
+// TODO:  probably no need for this.  Just use  D->m_peak, D->m_valley
+
 	if (amp >= D->alevel_mark_peak) {
-	  D->alevel_mark_peak = amp * D->quick_attack + D->alevel_mark_peak * (1. - D->quick_attack);
+	  D->alevel_mark_peak = amp * D->quick_attack + D->alevel_mark_peak * (1.0f - D->quick_attack);
 	}
 	else {
-	  D->alevel_mark_peak = amp * D->sluggish_decay + D->alevel_mark_peak * (1. - D->sluggish_decay);
+	  D->alevel_mark_peak = amp * D->sluggish_decay + D->alevel_mark_peak * (1.0f - D->sluggish_decay);
 	}
 
 	if (amp <= D->alevel_space_peak) {
-	  D->alevel_space_peak = amp * D->quick_attack + D->alevel_space_peak * (1. - D->quick_attack);
+	  D->alevel_space_peak = amp * D->quick_attack + D->alevel_space_peak * (1.0f - D->quick_attack);
 	}
 	else {
-	  D->alevel_space_peak = amp * D->sluggish_decay + D->alevel_space_peak * (1. - D->sluggish_decay);
+	  D->alevel_space_peak = amp * D->sluggish_decay + D->alevel_space_peak * (1.0f - D->sluggish_decay);
 	}
 
 /* 
@@ -349,28 +357,10 @@ void demod_9600_process_sample (int chan, int sam, struct demodulator_state_s *D
 
 	demod_out = agc (amp, D->agc_fast_attack, D->agc_slow_decay, &(D->m_peak), &(D->m_valley));
 
-
 // TODO: There is potential for multiple decoders with one filter.
 
 //dw_printf ("peak=%.2f valley=%.2f amp=%.2f norm=%.2f\n", D->m_peak, D->m_valley, amp, norm);
 
-	/* Throw in a little Hysteresis??? */
-	/* (Not to be confused with Hysteria.) */
-	/* Doesn't seem to have any value. */
-	/* Using a level of .02 makes things worse. */
-	/* Might want to experiment with this again someday. */
-
-
-//	if (demod_out > 0.03) {
-//	  demod_data = 1;
-//	}
-//	else if (demod_out < -0.03) {
-//	  demod_data = 0;
-//	} 
-//	else {
-//	  demod_data = D->slicer[subchan].prev_demod_data;
-//	}
-
 	if (D->num_slicers <= 1) {
 
 	  /* Normal case of one demodulator to one HDLC decoder. */
@@ -378,7 +368,7 @@ void demod_9600_process_sample (int chan, int sam, struct demodulator_state_s *D
 	  /* AGC should generally keep this around -1 to +1 range. */
 
 	  demod_data = demod_out > 0;
-	  nudge_pll (chan, subchan, 0, demod_data, D);
+	  nudge_pll (chan, subchan, 0, demod_out, D);
 	}
 	else {
 	  int slice;
@@ -386,76 +376,149 @@ void demod_9600_process_sample (int chan, int sam, struct demodulator_state_s *D
 	  /* Multiple slicers each feeding its own HDLC decoder. */
 
 	  for (slice=0; slice<D->num_slicers; slice++) {
-	    demod_data = demod_out > slice_point[slice];
-	    nudge_pll (chan, subchan, slice, demod_data, D);
+	    demod_data = demod_out - slice_point[slice] > 0;
+	    nudge_pll (chan, subchan, slice, demod_out - slice_point[slice], D);
 	  }
 	}
 
+	// demod_data is used only for debug out.
+	// suppress compiler warning about it not being used.
+	(void) demod_data;
+
+#if DEBUG4
+
+	if (chan == 0) {
+
+	  if (1) {
+	  //if (hdlc_rec_gathering (chan, subchan, slice)) {
+	    char fname[30];
+	    int slice = 0;
+
+	    if (demod_log_fp == NULL) {
+	      log_file_seq++;
+	      snprintf (fname, sizeof(fname), "demod/%04d.csv", log_file_seq);
+	      //if (log_file_seq == 1) mkdir ("demod", 0777);
+	      if (log_file_seq == 1) mkdir ("demod");
+
+	      demod_log_fp = fopen (fname, "w");
+	      text_color_set(DW_COLOR_DEBUG);
+	      dw_printf ("Starting demodulator log file %s\n", fname);
+	      fprintf (demod_log_fp, "Audio, Filtered,  Max,  Min, Normalized, Sliced, Clock\n");
+	    }
+
+	    fprintf (demod_log_fp, "%.3f, %.3f, %.3f, %.3f, %.3f, %d, %.2f\n",
+			fsam + 6,
+			amp + 4,
+			D->m_peak + 4,
+			D->m_valley + 4,
+			demod_out + 2,
+			demod_data + 2,
+			(D->slicer[slice].data_clock_pll & 0x80000000) ? .5 : .0);
+
+	    fflush (demod_log_fp);
+	  }
+	  else {
+	    if (demod_log_fp != NULL) {
+	      fclose (demod_log_fp);
+	      demod_log_fp = NULL;
+	    }
+	  }
+	}
+#endif
+
 } /* end demod_9600_process_sample */
 
 
+/*-------------------------------------------------------------------
+ *
+ * Name:        nudge_pll
+ *
+ * Purpose:	Update the PLL state for each audio sample.
+ *
+ *		(2) Descramble it.
+ *		(2) Recover clock and data.
+ *
+ * Inputs:	chan	- Audio channel.  0 for left, 1 for right.
+ *
+ *		subchan	- Which demodulator.  We could have several running in parallel.
+ *
+ *		slice	- Determines which Slicing level & HDLC decoder to use.
+ *
+ *		demod_out_f - Demodulator output, possibly shifted by slicing level
+ *				It will be compared with 0.0 to bit binary value out.
+ *
+ *		D	- Demodulator state for this channel / subchannel.
+ *
+ * Returns:	None
+ *
+ * Descripton:	A PLL is used to sample near the centers of the data bits.
+ *
+ *		D->data_clock_pll is a SIGNED 32 bit variable.
+ *		When it overflows from a large positive value to a negative value, we
+ *		sample a data bit from the demodulated signal.
+ *
+ *		Ideally, the the demodulated signal transitions should be near
+ *		zero we we sample mid way between the transitions.
+ *
+ *		Nudge the PLL by removing some small fraction from the value of
+ *		data_clock_pll, pushing it closer to zero.
+ *
+ *		This adjustment will never change the sign so it won't cause
+ *		any erratic data bit sampling.
+ *
+ *		If we adjust it too quickly, the clock will have too much jitter.
+ *		If we adjust it too slowly, it will take too long to lock on to a new signal.
+ *
+ *		I don't think the optimal value will depend on the audio sample rate
+ *		because this happens for each transition from the demodulator.
+ *
+ * Version 1.4:	Previously, we would always pull the PLL phase toward 0 after
+ *		after a zero crossing was detetected.  This adds extra jitter,
+ *		especially when the ratio of audio sample rate to baud is low.
+ *		Now, we interpolate between the two samples to get an estimate
+ *		on when the zero crossing happened.  The PLL is pulled toward
+ *		this point.
+ *
+ *		Results???  TBD
+ *
+ *--------------------------------------------------------------------*/
+
 __attribute__((hot))
-static void inline nudge_pll (int chan, int subchan, int slice, int demod_data, struct demodulator_state_s *D)
+inline static void nudge_pll (int chan, int subchan, int slice, float demod_out_f, struct demodulator_state_s *D)
 {
 
 /*
- * Next, a PLL is used to sample near the centers of the data bits.
- *
- * D->data_clock_pll is a SIGNED 32 bit variable.
- * When it overflows from a large positive value to a negative value, we 
- * sample a data bit from the demodulated signal.
- *
- * Ideally, the the demodulated signal transitions should be near
- * zero we we sample mid way between the transitions.
- *
- * Nudge the PLL by removing some small fraction from the value of 
- * data_clock_pll, pushing it closer to zero.
- * 
- * This adjustment will never change the sign so it won't cause
- * any erratic data bit sampling.
- *
- * If we adjust it too quickly, the clock will have too much jitter.
- * If we adjust it too slowly, it will take too long to lock on to a new signal.
- *
- * I don't think the optimal value will depend on the audio sample rate
- * because this happens for each transition from the demodulator.
- *
- * This was optimized for 1200 baud AFSK.  There might be some opportunity
- * for improvement here.
  */
 
 	D->slicer[slice].prev_d_c_pll = D->slicer[slice].data_clock_pll;
-	D->slicer[slice].data_clock_pll += D->pll_step_per_sample;
 
-	if (D->slicer[slice].data_clock_pll < 0 && D->slicer[slice].prev_d_c_pll > 0) {
+	// Perform the add as unsigned to avoid signed overflow error.
+	D->slicer[slice].data_clock_pll = (signed)((unsigned)(D->slicer[slice].data_clock_pll) + (unsigned)(D->pll_step_per_sample));
 
-	  /* Overflow. */
+	if ( D->slicer[slice].prev_d_c_pll > 1000000000 && D->slicer[slice].data_clock_pll < -1000000000) {
 
-/*
- * At this point, we need to descramble the data as
- * in hardware based designs by G3RUH and K9NG.
- *
- * Future Idea:  allow unscrambled baseband data.
- *
- * http://www.amsat.org/amsat/articles/g3ruh/109/fig03.gif
- */
-	  // Warning: 'descram' set but not used.
-	  // It's used in conditional debug code below.
-	  // descram =
-	  descramble (demod_data, &(D->slicer[slice].lfsr));
+	  /* Overflow.  Was large positive, wrapped around, now large negative. */
 
-	  hdlc_rec_bit (chan, subchan, slice, demod_data, 1, D->slicer[slice].lfsr);
+	  hdlc_rec_bit (chan, subchan, slice, demod_out_f > 0, 1, D->slicer[slice].lfsr);
 	}
 
-        if (demod_data != D->slicer[slice].prev_demod_data) {
+/*
+ * Zero crossing?
+ */
+        if ((D->slicer[slice].prev_demod_out_f < 0 && demod_out_f > 0) ||
+	    (D->slicer[slice].prev_demod_out_f > 0 && demod_out_f < 0)) {
 
 	  // Note:  Test for this demodulator, not overall for channel.
 
+	  float target = 0;
+
+	  target = D->pll_step_per_sample * demod_out_f / (demod_out_f - D->slicer[slice].prev_demod_out_f);
+
 	  if (hdlc_rec_gathering (chan, subchan, slice)) {
-	    D->slicer[slice].data_clock_pll = (int)(D->slicer[slice].data_clock_pll * D->pll_locked_inertia);
+	    D->slicer[slice].data_clock_pll = (int)(D->slicer[slice].data_clock_pll * D->pll_locked_inertia + target * (1.0f - D->pll_locked_inertia) );
 	  }
 	  else {
-	    D->slicer[slice].data_clock_pll = (int)(D->slicer[slice].data_clock_pll * D->pll_searching_inertia);
+	    D->slicer[slice].data_clock_pll = (int)(D->slicer[slice].data_clock_pll * D->pll_searching_inertia + target * (1.0f - D->pll_searching_inertia) );
 	  }
 	}
 
@@ -483,10 +546,10 @@ static void inline nudge_pll (int chan, int subchan, int slice, int demod_data,
 	    fprintf (demod_log_fp, "Audio, Peak, Valley, Demod, SData, Descram, Clock\n");
 	  }
 	  fprintf (demod_log_fp, "%.3f, %.3f, %.3f, %.3f, %.2f, %.2f, %.2f\n", 
-			0.5 * fsam + 3.5,  
-			0.5 * D->m_peak + 3.5,
-			0.5 * D->m_valley + 3.5,
-			0.5 * demod_out + 2.0,
+			0.5f * fsam + 3.5,
+			0.5f * D->m_peak + 3.5,
+			0.5f * D->m_valley + 3.5,
+			0.5f * demod_out + 2.0,
 			demod_data ? 1.35 : 1.0,  
 			descram ? .9 : .55,  
 			(D->data_clock_pll & 0x80000000) ? .1 : .45);
@@ -506,12 +569,9 @@ static void inline nudge_pll (int chan, int subchan, int slice, int demod_data,
  * Remember demodulator output (pre-descrambling) so we can compare next time
  * for the DPLL sync.
  */
-	D->slicer[slice].prev_demod_data = demod_data;
+	D->slicer[slice].prev_demod_out_f = demod_out_f;
 
 } /* end nudge_pll */
 
 
-
-
-
 /* end demod_9600.c */
