@@ -794,7 +794,8 @@ inline static void nudge_pll (int chan, int subchan, int slice, int demod_bits,
 
 	D->slicer[slice].prev_d_c_pll = D->slicer[slice].data_clock_pll;
 
-	D->slicer[slice].data_clock_pll += D->pll_step_per_sample;
+	// Perform the add as unsigned to avoid signed overflow error.
+	D->slicer[slice].data_clock_pll = (signed)((unsigned)(D->slicer[slice].data_clock_pll) + (unsigned)(D->pll_step_per_sample));
 
 	if (D->slicer[slice].data_clock_pll < 0 && D->slicer[slice].prev_d_c_pll >= 0) {
 
