@@ -1,7 +1,7 @@
 //
 //    This file is part of Dire Wolf, an amateur radio packet TNC.
 //
-//    Copyright (C) 2011-2014, 2015  John Langner, WB2OSZ
+//    Copyright (C) 2011-2014, 2015, 2017  John Langner, WB2OSZ
 //
 //    This program is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -49,28 +49,28 @@
  *	
  *		Commands from application recognized:
  *
- *			0	Data Frame	AX.25 frame in raw format.
+ *			_0	Data Frame	AX.25 frame in raw format.
  *
- *			1	TXDELAY		See explanation in xmit.c.
+ *			_1	TXDELAY		See explanation in xmit.c.
  *
- *			2	Persistence	"	"
+ *			_2	Persistence	"	"
  *
- *			3 	SlotTime	"	"
+ *			_3 	SlotTime	"	"
  *
- *			4	TXtail		"	"
+ *			_4	TXtail		"	"
  *						Spec says it is obsolete but Xastir
  *						sends it and we respect it.
  *
- *			5	FullDuplex	Ignored.  Always full duplex.
+ *			_5	FullDuplex	Ignored.
  *		
- *			6	SetHardware	TNC specific.  Ignored.
+ *			_6	SetHardware	TNC specific.
  *			
  *			FF	Return		Exit KISS mode.  Ignored.
  *
  *
  *		Messages sent to client application:
  *
- *			0	Data Frame	Received AX.25 frame in raw format.
+ *			_0	Data Frame	Received AX.25 frame in raw format.
  *
  *
  *		
@@ -91,14 +91,15 @@
 /*
  * Native Windows:	Use the Winsock interface.
  * Linux:		Use the BSD socket interface.
- * Cygwin:		Can use either one.
  */
 
 
+#include "direwolf.h"		// Sets _WIN32_WINNT for XP API level needed by ws2tcpip.h
+
+
 #if __WIN32__
 #include <winsock2.h>
-#define _WIN32_WINNT 0x0501
-#include <ws2tcpip.h>
+#include <ws2tcpip.h>  		// _WIN32_WINNT must be set to 0x0501 before including this
 #else 
 #include <stdlib.h>
 #include <sys/types.h>
@@ -115,11 +116,9 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <assert.h>
-
 #include <string.h>
 
 
-#include "direwolf.h"
 #include "tq.h"
 #include "ax25_pad.h"
 #include "textcolor.h"
@@ -128,19 +127,40 @@
 #include "kiss_frame.h"
 #include "xmit.h"
 
-
-static kiss_frame_t kf;		/* Accumulated KISS frame and state of decoder. */
-				// TODO: multiple instances if multiple KISS network clients!
+void hex_dump (unsigned char *p, int len);	// This should be in a .h file.
 
 
-static int client_sock;		/* File descriptor for socket for */
+/*
+ * Early on we allowed one AGW connection and one KISS TCP connection at a time.
+ * In version 1.1, we allowed multiple concurrent client apps to attach with the AGW network protocol.
+ * In Version 1.5, we do essentially the same here to allow multiple concurrent KISS TCP clients.
+ * The default is a limit of 3 client applications at the same time.
+ * You can increase the limit by changing the line below.
+ * A larger number consumes more resources so don't go crazy by making it larger than needed.
+ */
+
+#define MAX_NET_CLIENTS 3
+
+static int client_sock[MAX_NET_CLIENTS];
+				/* File descriptor for socket for */
 				/* communication with client application. */
 				/* Set to -1 if not connected. */
 				/* (Don't use SOCKET type because it is unsigned.) */
 
+static kiss_frame_t kf[MAX_NET_CLIENTS];
+				/* Accumulated KISS frame and state of decoder. */
 
-static void * connect_listen_thread (void *arg);
-static void * kissnet_listen_thread (void *arg);
+
+
+// TODO:  define in one place, use everywhere.
+#if __WIN32__
+#define THREAD_F unsigned __stdcall
+#else
+#define THREAD_F void *
+#endif
+
+static THREAD_F connect_listen_thread (void *arg);
+static THREAD_F kissnet_listen_thread (void *arg);
 
 
 
@@ -178,15 +198,17 @@ void kiss_net_set_debug (int n)
 
 void kissnet_init (struct misc_config_s *mc)
 {
+	int client;
+
 #if __WIN32__
 	HANDLE connect_listen_th;
-	HANDLE cmd_listen_th;
+	HANDLE cmd_listen_th[MAX_NET_CLIENTS];
 #else
 	pthread_t connect_listen_tid;
-	pthread_t cmd_listen_tid;
-#endif
+	pthread_t cmd_listen_tid[MAX_NET_CLIENTS];
 	int e;
-	int kiss_port = mc->kiss_port;
+#endif
+	int kiss_port = mc->kiss_port;		/* default 8001 but easily changed. */
 
 
 #if DEBUG
@@ -194,9 +216,11 @@ void kissnet_init (struct misc_config_s *mc)
 	dw_printf ("kissnet_init ( %d )\n", kiss_port);
 #endif
 
-	memset (&kf, 0, sizeof(kf));
 	
-	client_sock = -1;
+	for (client=0; client<MAX_NET_CLIENTS; client++) {
+	  client_sock[client] = -1;
+	  memset (&(kf[client]), 0, sizeof(kf[client]));
+	}
 
 	if (kiss_port == 0) {
 	  text_color_set(DW_COLOR_INFO);
@@ -205,10 +229,10 @@ void kissnet_init (struct misc_config_s *mc)
 	}
 	
 /*
- * This waits for a client to connect and sets client_sock.
+ * This waits for a client to connect and sets client_sock[n].
  */
 #if __WIN32__
-	connect_listen_th = _beginthreadex (NULL, 0, connect_listen_thread, (void *)kiss_port, 0, NULL);
+	connect_listen_th = (HANDLE)_beginthreadex (NULL, 0, connect_listen_thread, (void *)kiss_port, 0, NULL);
 	if (connect_listen_th == NULL) {
 	  text_color_set(DW_COLOR_ERROR);
 	  dw_printf ("Could not create KISS socket connect listening thread\n");
@@ -224,23 +248,30 @@ void kissnet_init (struct misc_config_s *mc)
 #endif
 
 /*
- * This reads messages from client when client_sock is valid.
+ * These read messages from client when client_sock[n] is valid.
+ * Currently we start up a separate thread for each potential connection.
+ * Possible later refinement.  Start one now, others only as needed.
  */
+	for (client = 0; client < MAX_NET_CLIENTS; client++) {
+
 #if __WIN32__
-	cmd_listen_th = _beginthreadex (NULL, 0, kissnet_listen_thread, NULL, 0, NULL);
-	if (cmd_listen_th == NULL) {
-	  text_color_set(DW_COLOR_ERROR);
-	  dw_printf ("Could not create KISS socket command listening thread\n");
-	  return;
-	}
+	  cmd_listen_th[client] = (HANDLE)_beginthreadex (NULL, 0, kissnet_listen_thread, (void*)client, 0, NULL);
+	  if (cmd_listen_th[client] == NULL) {
+	    text_color_set(DW_COLOR_ERROR);
+	    dw_printf ("Could not create KISS command listening thread for client %d\n", client);
+	    return;
+	  }
 #else
-	e = pthread_create (&cmd_listen_tid, NULL, kissnet_listen_thread, NULL);
-	if (e != 0) {
-	  text_color_set(DW_COLOR_ERROR);
-	  perror("Could not create KISS socket command listening thread");
-	  return;
-	}
+	  e = pthread_create (&(cmd_listen_tid[client]), NULL, kissnet_listen_thread, (void *)(long)client);
+	  if (e != 0) {
+	    text_color_set(DW_COLOR_ERROR);
+	    dw_printf ("Could not create KISS command listening thread for client %d\n", client);
+	    // Replace add perror with better message handling.
+	    perror("");
+	    return;
+	  }
 #endif
+	}
 }
 
 
@@ -263,7 +294,7 @@ void kissnet_init (struct misc_config_s *mc)
  *
  *--------------------------------------------------------------------*/
 
-static void * connect_listen_thread (void *arg)
+static THREAD_F connect_listen_thread (void *arg)
 {
 #if __WIN32__
 
@@ -284,7 +315,7 @@ static void * connect_listen_thread (void *arg)
 	if (err != 0) {
 	    text_color_set(DW_COLOR_ERROR);
 	    dw_printf("WSAStartup failed: %d\n", err);
-	    return (NULL);
+	    return (0);
 	}
 
 	if (LOBYTE(wsadata.wVersion) != 2 || HIBYTE(wsadata.wVersion) != 2) {
@@ -292,7 +323,7 @@ static void * connect_listen_thread (void *arg)
           dw_printf("Could not find a usable version of Winsock.dll\n");
           WSACleanup();
 	  //sleep (1);
-          return (NULL);
+          return (0);
 	}
 
 	memset (&hints, 0, sizeof(hints));
@@ -307,14 +338,14 @@ static void * connect_listen_thread (void *arg)
 	    dw_printf("getaddrinfo failed: %d\n", err);
 	    //sleep (1);
 	    WSACleanup();
-	    return (NULL);
+	    return (0);
 	}
 
 	listen_sock= socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
 	if (listen_sock == INVALID_SOCKET) {
 	  text_color_set(DW_COLOR_ERROR);
 	  dw_printf ("connect_listen_thread: Socket creation failed, err=%d", WSAGetLastError());
-	  return (NULL);
+	  return (0);
 	}
 
 #if DEBUG
@@ -331,7 +362,7 @@ static void * connect_listen_thread (void *arg)
           freeaddrinfo(ai);
           closesocket(listen_sock);
           WSACleanup();
-          return (NULL);
+          return (0);
         }
 
 	freeaddrinfo(ai);
@@ -343,35 +374,50 @@ static void * connect_listen_thread (void *arg)
 
  	while (1) {
   	 
-	  while (client_sock > 0) {
-	    SLEEP_SEC(1);			/* Already connected.  Try again later. */
+	  int client;
+	  int c;
+
+	  client = -1;
+	  for (c = 0; c < MAX_NET_CLIENTS && client < 0; c++) {
+	    if (client_sock[c] <= 0) {
+	      client = c;
+	    }
 	  }
 
-#define QUEUE_SIZE 5
+/*
+ * Listen for connection if we have not reached maximum.
+ */
+	  if (client >= 0) {
 
-	  if(listen(listen_sock,QUEUE_SIZE) == SOCKET_ERROR)
-	  {
-	    text_color_set(DW_COLOR_ERROR);
-            dw_printf("Listen failed with error: %d\n", WSAGetLastError());
-	    return (NULL);
-	  }
+	    if(listen(listen_sock, MAX_NET_CLIENTS) == SOCKET_ERROR)
+	    {
+	      text_color_set(DW_COLOR_ERROR);
+              dw_printf("Listen failed with error: %d\n", WSAGetLastError());
+	      return (0);
+	    }
 	
-	  text_color_set(DW_COLOR_INFO);
-          dw_printf("Ready to accept KISS client application on port %s ...\n", kiss_port_str);
+	    text_color_set(DW_COLOR_INFO);
+            dw_printf("Ready to accept KISS TCP client application %d on port %s ...\n", client, kiss_port_str);
          
-          client_sock = accept(listen_sock, NULL, NULL);
+            client_sock[client] = accept(listen_sock, NULL, NULL);
 
-	  if (client_sock == -1) {
-	    text_color_set(DW_COLOR_ERROR);
-            dw_printf("Accept failed with error: %d\n", WSAGetLastError());
-            closesocket(listen_sock);
-            WSACleanup();
-            return (NULL);
-          }
+	    if (client_sock[client] == -1) {
+	      text_color_set(DW_COLOR_ERROR);
+              dw_printf("Accept failed with error: %d\n", WSAGetLastError());
+              closesocket(listen_sock);
+              WSACleanup();
+              return (0);
+            }
 
-	  text_color_set(DW_COLOR_INFO);
-	  dw_printf("\nConnected to KISS client application ...\n\n");
+	    text_color_set(DW_COLOR_INFO);
+	    dw_printf("\nAttached to KISS TCP client application %d ...\n\n", client);
 
+	    // Reset the state and buffer.
+	    memset (&(kf[client]), 0, sizeof(kf[client]));
+	  }
+	  else {
+	    SLEEP_SEC(1);	/* wait then check again if more clients allowed. */
+	  }
  	}
 
 
@@ -421,32 +467,44 @@ static void * connect_listen_thread (void *arg)
 
 #if DEBUG
 	text_color_set(DW_COLOR_DEBUG);
- 	dw_printf("opened KISS socket as fd (%d) on port (%d) for stream i/o\n", listen_sock, ntohs(sockaddr.sin_port) );
+ 	dw_printf("opened KISS TCP socket as fd (%d) on port (%d) for stream i/o\n", listen_sock, ntohs(sockaddr.sin_port) );
 #endif
 
  	while (1) {
-  	 
-	  while (client_sock > 0) {
-	    SLEEP_SEC(1);			/* Already connected.  Try again later. */
+
+	  int client;
+	  int c;
+
+	  client = -1;
+	  for (c = 0; c < MAX_NET_CLIENTS && client < 0; c++) {
+	    if (client_sock[c] <= 0) {
+	      client = c;
+	    }
 	  }
 
-#define QUEUE_SIZE 5
+	  if (client >= 0) {
 
-	  if(listen(listen_sock,QUEUE_SIZE) == -1)
-	  {
-	    text_color_set(DW_COLOR_ERROR);
-	    perror ("connect_listen_thread: Listen failed");
-	    return (NULL);
-	  }
+	    if(listen(listen_sock,MAX_NET_CLIENTS) == -1)
+	    {
+	      text_color_set(DW_COLOR_ERROR);
+	      perror ("connect_listen_thread: Listen failed");
+	      return (NULL);
+	    }
 	
-	  text_color_set(DW_COLOR_INFO);
-          dw_printf("Ready to accept KISS client application on port %d ...\n", kiss_port);
+	    text_color_set(DW_COLOR_INFO);
+            dw_printf("Ready to accept KISS TCP client application %d on port %d ...\n", client, kiss_port);
          
-          client_sock = accept(listen_sock, (struct sockaddr*)(&sockaddr),&sockaddr_size);
+            client_sock[client] = accept(listen_sock, (struct sockaddr*)(&sockaddr),&sockaddr_size);
 
-	  text_color_set(DW_COLOR_INFO);
-	  dw_printf("\nConnected to KISS client application ...\n\n");
+	    text_color_set(DW_COLOR_INFO);
+	    dw_printf("\nAttached to KISS TCP client application %d...\n\n", client);
 
+	    // Reset the state and buffer.
+	    memset (&(kf[client]), 0, sizeof(kf[client]));
+	  }
+	  else {
+	    SLEEP_SEC(1);	/* wait then check again if more clients allowed. */
+	  }
  	}
 #endif
 }
@@ -464,157 +522,148 @@ static void * connect_listen_thread (void *arg)
  * Inputs:	chan		- Channel number where packet was received.
  *				  0 = first, 1 = second if any.
  *
+// TODO: add kiss_cmd
+ *
  *		fbuf		- Address of raw received frame buffer
  *				  or a text string.
  *
- *		flen		- Number of bytes for AX.25 frame.
- *				  or -1 for a text string.
- *		
+ *		kiss_cmd	- Usually KISS_CMD_DATA_FRAME but we can also have
+ *				  KISS_CMD_SET_HARDWARE when responding to a query.
  *
- * Description:	Send message to client if connected.
+ *		flen		- Number of bytes for AX.25 frame.
+ *				  When called from kiss_rec_byte, flen will be -1
+ *				  indicating a text string rather than frame content.
+ *				  This is used to fake out an application that thinks
+ *				  it is using a traditional TNC and tries to put it
+ *				  into KISS mode.
+ *
+ *		tcpclient	- It is possible to have more than client attached
+ *				  at the same time with TCP KISS.
+ *				  When a frame is received from the radio we want it
+ *				  to go to all of the clients.  In this case specify -1.
+ *				  When responding to a command from the client, we want
+ *				  to send only to that one client app.  In this case
+ *				  use the value 0 .. MAX_NET_CLIENTS-1.
+ *
+ * Description:	Send message to client(s) if connected.
  *		Disconnect from client, and notify user, if any error.
  *
  *--------------------------------------------------------------------*/
 
 
-void kissnet_send_rec_packet (int chan, unsigned char *fbuf, int flen)
+void kissnet_send_rec_packet (int chan, int kiss_cmd, unsigned char *fbuf, int flen, int tcpclient)
 {
 	unsigned char kiss_buff[2 * AX25_MAX_PACKET_LEN];
 	int kiss_len;
-	int j;
 	int err;
+	int first, last, client;
 
+// Something received over the radio would be sent to all attached clients.
+// However, there are times we want to send a response only to a particular client.
+// In the case of a serial port or pseudo terminal, there is only one potential client.
+// so the response would be sent to only one place.  A new parameter has been added for this.
 
-	if (client_sock == -1) {
-	  return;
+	if (tcpclient >= 0 && tcpclient < MAX_NET_CLIENTS) {
+	  first = tcpclient;
+	  last = tcpclient;
 	}
-	if (flen < 0) {
-	  flen = strlen((char*)fbuf);
-	  if (kiss_debug) {
-	    kiss_debug_print (TO_CLIENT, "Fake command prompt", fbuf, flen);
-	  }
-	  strlcpy ((char *)kiss_buff, (char *)fbuf, sizeof(kiss_buff));
-	  kiss_len = strlen((char *)kiss_buff);
+	else if (tcpclient == -1) {
+	  first = 0;
+	  last = MAX_NET_CLIENTS - 1;
 	}
 	else {
-
-
-	  unsigned char stemp[AX25_MAX_PACKET_LEN + 1];
-	 
-	  assert (flen < sizeof(stemp));
-
-	  stemp[0] = (chan << 4) + 0;
-	  memcpy (stemp+1, fbuf, flen);
-
-	  if (kiss_debug >= 2) {
-	    /* AX.25 frame with the CRC removed. */
-	    text_color_set(DW_COLOR_DEBUG);
-	    dw_printf ("\n");
-	    dw_printf ("Packet content before adding KISS framing and any escapes:\n");
-	    hex_dump (fbuf, flen);
-	  }
-
-	  kiss_len = kiss_encapsulate (stemp, flen+1, kiss_buff);
-
-	  /* This has the escapes and the surrounding FENDs. */
-
-	  if (kiss_debug) {
-	    kiss_debug_print (TO_CLIENT, NULL, kiss_buff, kiss_len);
-	  }
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf ("KISS TCP: Internal error, kissnet_send_rec_packet, tcpclient = %d.\n", tcpclient);
+	  return;
 	}
 
+
+	for (client = first; client <= last; client++) {
+
+	  if (client_sock[client] != -1) {
+
+	    if (flen < 0) {
+
+// A client app might think it is attached to a traditional TNC.
+// It might try sending commands over and over again trying to get the TNC into KISS mode.
+// We recognize this attempt and send it something to keep it happy.
+
+	      text_color_set(DW_COLOR_ERROR);
+	      dw_printf ("KISS TCP: Something unexpected from client application.\n");
+	      dw_printf ("Is client app treating this like an old TNC with command mode?\n");
+	      dw_printf ("This can be caused by the application sending commands to put a\n");
+	      dw_printf ("traditional TNC into KISS mode.  It is usually a harmless warning.\n");
+	      dw_printf ("For best results, configure for a KISS-only TNC to avoid this.\n");
+	      dw_printf ("In the case of APRSISCE/32, use \"Simply(KISS)\" rather than \"KISS.\"\n");
+
+	      flen = strlen((char*)fbuf);
+	      if (kiss_debug) {
+	        kiss_debug_print (TO_CLIENT, "Fake command prompt", fbuf, flen);
+	      }
+	      strlcpy ((char *)kiss_buff, (char *)fbuf, sizeof(kiss_buff));
+	      kiss_len = strlen((char *)kiss_buff);
+	    }
+	    else {
+	      unsigned char stemp[AX25_MAX_PACKET_LEN + 1];
+
+	      assert (flen < (int)(sizeof(stemp)));
+
+	      stemp[0] = (chan << 4) | kiss_cmd;
+	      memcpy (stemp+1, fbuf, flen);
+
+	      if (kiss_debug >= 2) {
+	        /* AX.25 frame with the CRC removed. */
+	        text_color_set(DW_COLOR_DEBUG);
+	        dw_printf ("\n");
+	        dw_printf ("Packet content before adding KISS framing and any escapes:\n");
+	        hex_dump (fbuf, flen);
+	      }
+
+	      kiss_len = kiss_encapsulate (stemp, flen+1, kiss_buff);
+
+	      /* This has the escapes and the surrounding FENDs. */
+
+	      if (kiss_debug) {
+	        kiss_debug_print (TO_CLIENT, NULL, kiss_buff, kiss_len);
+	      }
+	    }
+
 #if __WIN32__	
-        err = send (client_sock, (char*)kiss_buff, kiss_len, 0);
-	if (err == SOCKET_ERROR)
-	{
-	  text_color_set(DW_COLOR_ERROR);
-	  dw_printf ("\nError %d sending message to KISS client application.  Closing connection.\n\n", WSAGetLastError());
-	  closesocket (client_sock);
-	  client_sock = -1;
-	  WSACleanup();
-	}
+            err = SOCK_SEND(client_sock[client], (char*)kiss_buff, kiss_len);
+	    if (err == SOCKET_ERROR)
+	    {
+	      text_color_set(DW_COLOR_ERROR);
+	      dw_printf ("\nError %d sending message to KISS client %d application.  Closing connection.\n\n", WSAGetLastError(), client);
+	      closesocket (client_sock[client]);
+	      client_sock[client] = -1;
+	      WSACleanup();
+	    }
 #else
-        err = write (client_sock, kiss_buff, kiss_len);
-	if (err <= 0)
-	{
-	  text_color_set(DW_COLOR_ERROR);
-	  dw_printf ("\nError sending message to KISS client application.  Closing connection.\n\n");
-	  close (client_sock);
-	  client_sock = -1;    
-	}
+            err = SOCK_SEND (client_sock[client], kiss_buff, kiss_len);
+	    if (err <= 0)
+	    {
+	      text_color_set(DW_COLOR_ERROR);
+	      dw_printf ("\nError sending message to KISS client %d application.  Closing connection.\n\n", client);
+	      close (client_sock[client]);
+	      client_sock[client] = -1;
+	    }
 #endif
+	  }
+	}
 	
 } /* end kissnet_send_rec_packet */
 
 
 
-/*-------------------------------------------------------------------
- *
- * Name:        read_from_socket
- *
- * Purpose:     Read from socket until we have desired number of bytes.
- *
- * Inputs:	fd		- file descriptor.
- *		ptr		- address where data should be placed.
- *		len		- desired number of bytes.
- *
- * Description:	Just a wrapper for the "read" system call but it should
- *		never return fewer than the desired number of bytes.
- *
- * 		Not really needed for KISS because we are dealing with
- *		a stream of bytes rather than message blocks.
- *
- *--------------------------------------------------------------------*/
-
-static int read_from_socket (int fd, char *ptr, int len)
-{
-	int got_bytes = 0;
-
-#if DEBUG
-	text_color_set(DW_COLOR_DEBUG);
-	dw_printf ("read_from_socket (%d, %p, %d)\n", fd, ptr, len);
-#endif
-	while (got_bytes < len) {
-	  int n;
-
-#if __WIN32__
-
-//TODO: any flags for send/recv?
-
-	  n = recv (fd, ptr + got_bytes, len - got_bytes, 0);
-#else
-	  n = read (fd, ptr + got_bytes, len - got_bytes);
-#endif
-
-#if DEBUG
-	  text_color_set(DW_COLOR_DEBUG);
-	  dw_printf ("read_from_socket: n = %d\n", n);
-#endif
-	  if (n <= 0) {
-	    return (n);
-	  }
-
-	  got_bytes += n;
-	}
-	assert (got_bytes >= 0 && got_bytes <= len);
-
-#if DEBUG
-	text_color_set(DW_COLOR_DEBUG);
-	dw_printf ("read_from_socket: return %d\n", got_bytes);
-#endif
-	return (got_bytes);
-}
-
-
 /*-------------------------------------------------------------------
  *
  * Name:        kissnet_listen_thread
  *
  * Purpose:     Wait for KISS messages from an application.
  *
- * Inputs:	arg		- Not used.
+ * Inputs:	arg		- client number, 0 .. MAX_NET_CLIENTS-1
  *
- * Outputs:	client_sock	- File descriptor for communicating with client app.
+ * Outputs:	client_sock[n]	- File descriptor for communicating with client app.
  *
  * Description:	Process messages from the client application.
  *		Note that the client can go away and come back again and
@@ -626,20 +675,20 @@ static int read_from_socket (int fd, char *ptr, int len)
 /* Return one byte (value 0 - 255) */
 
 
-static int kiss_get (void)
+static int kiss_get (int client)
 {
 	unsigned char ch;
 	int n;
 
 	while (1) {
 
-	  while (client_sock <= 0) {
+	  while (client_sock[client] <= 0) {
 	    SLEEP_SEC(1);			/* Not connected.  Try again later. */
 	  }
 
 	  /* Just get one byte at a time. */
 
-	  n = read_from_socket (client_sock, (char *)(&ch), 1);
+	  n = SOCK_RECV (client_sock[client], (char *)(&ch), 1);
 
 	  if (n == 1) {
 #if DEBUG9
@@ -659,33 +708,55 @@ static int kiss_get (void)
 	  }
 
           text_color_set(DW_COLOR_ERROR);
-	  dw_printf ("\nError reading KISS byte from clent application.  Closing connection.\n\n");
+	  dw_printf ("\nKISS client application %d has gone away.\n\n", client);
 #if __WIN32__
-	  closesocket (client_sock);
+	  closesocket (client_sock[client]);
 #else
-	  close (client_sock);
+	  close (client_sock[client]);
 #endif
-	  client_sock = -1;
+	  client_sock[client] = -1;
 	}
 }
 
 
 
-static void * kissnet_listen_thread (void *arg)
+static THREAD_F kissnet_listen_thread (void *arg)
 {
 	unsigned char ch;
 			
+
+	int client = (int)(long)arg;
+
 #if DEBUG
 	text_color_set(DW_COLOR_DEBUG);
-	dw_printf ("kissnet_listen_thread ( socket = %d )\n", client_sock);
+	dw_printf ("kissnet_listen_thread ( client = %d, socket fd = %d )\n", client, client_sock[client]);
 #endif
 
+	assert (client >= 0 && client < MAX_NET_CLIENTS);
+
+
+// So why is kissnet_send_rec_packet mentioned here for incoming from the client app?
+// The logic exists for the serial port case where the client might think it is
+// attached to a traditional TNC.  It might try sending commands over and over again
+// trying to get the TNC into KISS mode.  To keep it happy, we recognize this attempt
+// and send it something to keep it happy.
+// In the case of a serial port or pseudo terminal, there is only one potential client
+// so the response would be sent to only one place.
+// Starting in version 1.5, this now can have multiple attached clients.  We wouldn't
+// want to send the response to all of them.   Actually, we should be providing only
+// "Simply KISS" as some call it.
+
+
 	while (1) {
-	  ch = kiss_get();
-	  kiss_rec_byte (&kf, ch, kiss_debug, kissnet_send_rec_packet);
+	  ch = kiss_get(client);
+	  kiss_rec_byte (&(kf[client]), ch, kiss_debug, client, kissnet_send_rec_packet);
 	}  
 
-	return (NULL);	/* to suppress compiler warning. */
+#if __WIN32__
+	return(0);
+#else
+	return (THREAD_F) 0;	/* Unreachable but avoids compiler warning. */
+#endif
 
 } /* end kissnet_listen_thread */
 
