@@ -143,9 +143,10 @@ static struct t_metadata_s * t_get_metadata (char *station)
 	for (p = md_list_head; p != NULL; p = p->pnext) {
 	  if (strcmp(station, p->station) == 0) {
 
-	    assert (p->magic1 == MAGIC1);
-	    assert (p->magic2 == MAGIC2);
-
+	    if (p->magic1 != MAGIC1 || p->magic2 != MAGIC2) {
+	      text_color_set(DW_COLOR_ERROR);
+	      dw_printf("Internal error: REPORT THIS! Bad magic values %s %d\n", __func__, __LINE__);
+	    }
 	    return (p);
 	  }
 	}
@@ -252,7 +253,10 @@ static int t_ndp (char *str)
  *			KB1GKN-10>APRX27,UNCAN,WIDE1*:T#491,4.9,0.3,25.0,0.0,1.0,00000000
  *
  *		Not integers.  Not fixed width fields.
- *		We will accept these but issue a warning that others might not.
+ *
+ *		Originally I printed a warning if values were not in range of 000 to 255
+ *		but later took it out because no one pays attention to that original
+ *		restriction anymore.
  *		
  *--------------------------------------------------------------------*/
 
@@ -282,8 +286,10 @@ void telemetry_data_original (char *station, char *info, int quiet, char *output
 
 	pm = t_get_metadata(station);
 
-	assert (pm->magic1 == MAGIC1);
-	assert (pm->magic2 == MAGIC2);
+	if (pm->magic1 != MAGIC1 || pm->magic2 != MAGIC2) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf("Internal error: REPORT THIS! Bad magic values %s %d\n", __func__, __LINE__);
+	}
 
 	seq = 0;
 	for (n = 0; n < T_NUM_ANALOG; n++) {
@@ -297,13 +303,13 @@ void telemetry_data_original (char *station, char *info, int quiet, char *output
 	if (strncmp(info, "T#", 2) != 0) {
 	  if ( ! quiet) {
 	    text_color_set(DW_COLOR_ERROR);
-	    dw_printf("Error: Information part of telemetry packet must begin with \"#\"\n");
+	    dw_printf("Error: Information part of telemetry packet must begin with \"T#\"\n");
 	  }
 	  return;	
 	}
 
 /*
- * Make a copy of the input string because this will alter it.
+ * Make a copy of the input string (excluding T#) because this will alter it.
  * Remove any trailing CR/LF.
  */
 
@@ -315,6 +321,15 @@ void telemetry_data_original (char *station, char *info, int quiet, char *output
 
 	next = stemp;
 	p = strsep(&next,",");
+
+	if (p == NULL) {
+	  if ( ! quiet) {
+	    text_color_set(DW_COLOR_ERROR);
+	    dw_printf("Nothing after \"T#\" for telemetry data.\n");
+	  }
+	  return;
+	}
+
 	seq = atoi(p);
 	n = 0;
 	while ((p = strsep(&next,",")) != NULL) {
@@ -470,8 +485,10 @@ void telemetry_data_base91 (char *station, char *cdata, char *output, size_t out
 
 	pm = t_get_metadata(station);
 
-	assert (pm->magic1 == MAGIC1);
-	assert (pm->magic2 == MAGIC2);
+	if (pm->magic1 != MAGIC1 || pm->magic2 != MAGIC2) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf("Internal error: REPORT THIS! Bad magic values %s %d\n", __func__, __LINE__);
+	}
 
 	seq = 0;
 	for (n = 0; n < T_NUM_ANALOG; n++) {
@@ -576,8 +593,11 @@ void telemetry_name_message (char *station, char *msg)
 	} 
 
 	pm = t_get_metadata(station);
-	assert (pm->magic1 == MAGIC1);
-	assert (pm->magic2 == MAGIC2);
+
+	if (pm->magic1 != MAGIC1 || pm->magic2 != MAGIC2) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf("Internal error: REPORT THIS! Bad magic values %s %d\n", __func__, __LINE__);
+	}
 
 	next = stemp;
 
@@ -652,8 +672,11 @@ void telemetry_unit_label_message (char *station, char *msg)
 	} 
 
 	pm = t_get_metadata(station);
-	assert (pm->magic1 == MAGIC1);
-	assert (pm->magic2 == MAGIC2);
+
+	if (pm->magic1 != MAGIC1 || pm->magic2 != MAGIC2) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf("Internal error: REPORT THIS! Bad magic values %s %d\n", __func__, __LINE__);
+	}
 
 	next = stemp;
 
@@ -729,8 +752,11 @@ void telemetry_coefficents_message (char *station, char *msg, int quiet)
 	} 
 
 	pm = t_get_metadata(station);
-	assert (pm->magic1 == MAGIC1);
-	assert (pm->magic2 == MAGIC2);
+
+	if (pm->magic1 != MAGIC1 || pm->magic2 != MAGIC2) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf("Internal error: REPORT THIS! Bad magic values %s %d\n", __func__, __LINE__);
+	}
 
 	next = stemp;
 
@@ -809,8 +835,11 @@ void telemetry_bit_sense_message (char *station, char *msg, int quiet)
 #endif
 
 	pm = t_get_metadata(station);
-	assert (pm->magic1 == MAGIC1);
-	assert (pm->magic2 == MAGIC2);
+
+	if (pm->magic1 != MAGIC1 || pm->magic2 != MAGIC2) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf("Internal error: REPORT THIS! Bad magic values %s %d\n", __func__, __LINE__);
+	}
 
 	if (strlen(msg) < 8) {
 	  if ( ! quiet) {
@@ -919,8 +948,11 @@ static void t_data_process (struct t_metadata_s *pm, int seq, float araw[T_NUM_A
 
 
 	assert (pm != NULL);
-	assert (pm->magic1 == MAGIC1);
-	assert (pm->magic2 == MAGIC2);
+
+	if (pm->magic1 != MAGIC1 || pm->magic2 != MAGIC2) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf("Internal error: REPORT THIS! Bad magic values %s %d\n", __func__, __LINE__);
+	}
 
 	strlcpy (output, "", outputsize);
 
